// IDA.idc
//
//   Delphi
//
// :
// . unit System TObject 
//
// 24.03.2005

#include <idc.idc>

//
//
//
//
#define TYPEINFOENUM "TypeInfoStructures"
#define RTTIENUM "RTTIStructures"
#define OFFSETSTIENUM "OffsetsTIEnum"
#define OFFSETSRTTIENUM "OffsetsRTTIEnum"

// consts for TypeInfo
#define tkUnknown       0
#define tkInteger       1
#define tkChar          2
#define tkEnumeration   3
#define tkFloat	        4
#define tkString        5
#define tkSet 	        6
#define tkClass         7
#define tkMethod        8
#define tkWChar		9
#define tkLString       10
#define tkWString       11
#define tkVariant       12
#define tkArray	        13
#define tkRecord        14
#define tkInterface	15
#define tkInt64		16
#define tkDynArray	17

// consts for OrdType
#define otSByte         0
#define otUByte         1
#define otSWord         2
#define otUWord         3
#define otSLong         4

//consts for FloatType
#define ftSingle        0
#define ftDouble        1
#define ftExtended      2
#define ftComp          3
#define ftCurr          4

// Interface Flags
#define ifHasGuid         0
#define ifDispInterface   1
#define ifDispatch        2

// Method Kinds
#define mkProcedure       0
#define mkFunction        1
#define mkConstructor     2
#define mkDestructor      3
#define mkClassProcedure  4
#define mkClassFunction   5
#define mkSafeProcedure   6
#define mkSafeFunction    7

// Parameter Flags
#define pfVar             0
#define pfConst           1
#define pfArray           2
#define pfAddress         3
#define pfReference       4
#define pfOut             5

static Abs(Value)
{
	if (Value>0)
		return Value;
	else
		return -Value;
}


//*************************************************************************************
//
//  Functions for operation with byte, word and dword
//
//*************************************************************************************

//
// Function ReMakeByte
//   makes byte on address Address
//   return value of byte on address Address
//
static ReMakeByte ( Address )
{
  MakeUnkn ( Address, 0 );
  MakeByte ( Address );
  return Byte ( Address );
}

//
// Function ReMakeWord
//   makes word on address Address
//   return value of word on address Address
//
static ReMakeWord ( Address )
{
  MakeUnkn ( Address, 0 );
  MakeUnkn ( Address + 1, 0 );
  MakeWord ( Address );
  return Word ( Address );
}

//
// Function ReMakeDword
//   makes dword on address Address
//   return value of dword on address Address
//
static ReMakeDword ( Address )
{
  MakeUnkn ( Address, 0 );
  MakeUnkn ( Address + 1, 0 );
  MakeUnkn ( Address + 2, 0 );
  MakeUnkn ( Address + 3, 0 );
  MakeDword ( Address );
  return Dword ( Address );
}

//
// Function ReMakeQword
//   makes dword on address Address
//   return value of dword on address Address
//
static ReMakeQword ( Address )
{
  MakeUnkn ( Address, 0 );
  MakeUnkn ( Address + 1, 0 );
  MakeUnkn ( Address + 2, 0 );
  MakeUnkn ( Address + 3, 0 );
  MakeUnkn ( Address + 4, 0 );
  MakeUnkn ( Address + 5, 0 );
  MakeUnkn ( Address + 6, 0 );
  MakeUnkn ( Address + 7, 0 );
  MakeQword ( Address );
  return ; // Qword ( Address );
}

//*************************************************************************************
//
//  Functions for operation with Str
//
//
//*************************************************************************************

//
//  Function ReMakeStr
//    makes Str to data on Address with length equal Length
//   return 1 if success
//          0 if failure
static ReMakeStr ( Address, Length )
{
  auto I;
  for ( I = 0; I < Length; I++ )
    MakeUnkn ( Address + I, 0 );
  return MakeStr( Address , Address + Length );
}

//
//  Function MakePasStr
//    makes Pas-style Str
//   return 1 if success
//          0 if failure
static MakePasStr(Address)
{
auto Length;
	Length=ReMakeByte(Address);
	return ReMakeStr(Address,Length+1);
}

//
//  Function MakeCurPasStr
//    makes Pas-style Str
//   return 1 if success
//          0 if failure
static MakeCurPasStr ()
{
  return MakePasStr( ScreenEA () );
}

//
//  Function GetPasStr
//    get Pas-style Str
//  return Str
//
static GetPasStr( Address )
{
  auto Length, Result;
  MakePasStr ( Address ) ;
  Length = Byte ( Address++ );
  Result = "";
  for ( ; Length; Length-- )
    Result = Result + Byte( Address++ );
  return Result;
}

//
//  Function CopyPasStr
//    get Pas-style Str
//  return Str
//
static CopyPasStr( Address )
{
  auto Length, Result;
  Length = Byte ( Address++ );
  Result = "";
  for ( ; Length; Length-- )
    Result = Result + Byte( Address++ );
  return Result;
}

//
//  Function GetPasStr
//    get address after end of Pas-style Str
//  return Address
//
static GetAddrAfterStr(Address)
{
	Address=Address+Byte(Address)+1;
	return Address ;
}

//*************************************************************************************
//
//  Function for operation with offset
//
//*************************************************************************************

//
// Function MakeOffset
//   makes offset to data on Address
//   return offset to data
//

static MakeOffset ( Address )
{
  auto Offset;
  Offset = ReMakeDword ( Address );
  if ( Offset != 0 )
    OpOff( Address, 0, 0 );
  return Offset ;
}

//
// Function MakeFuncOffset
//   makes dword and offset to a function
//   return 1 if success
//          0 if failure
static MakeFuncOffset ( Address )
{
  auto FuncOffset ;
  FuncOffset = MakeOffset ( Address ) ;
  if ( FuncOffset == 0 )
    return 0 ;
  MakeUnkn ( FuncOffset, 1 ) ;
  if ( MakeFunction ( FuncOffset, BADADDR ) == 0 )
    return 0 ;
  add_dref ( Address, FuncOffset, 0 );
  return 1 ;
}

// Function MakeNamFuncOff
//   makes named function on Address
//   return 1 if success
//          0 if failure
static MakeNamFuncOff ( Address, Name )
{
  if ( !MakeFuncOffset ( Address ) )
    return 0 ;
  MakeName ( Dword ( Address ), Name ) ;
  return 1 ;
}


//***********************************************************************
//
//  Function for operation with enum
//
//***********************************************************************
//
// Function InitEnum
//   creates enum with name EnumName
//   return ID of enum
//
static InitEnum(EnumName, Asked)
{
  auto ID;
  auto Deleted;
  Deleted=0;
  ID=AddEnum(0, EnumName, 0);
  if (ID != -1)
    return ID;
  if (Asked)
  {
    Deleted=AskYN(0 ,"Enum with name '"+EnumName +"' already exist."+
                     " Do you want delete enum?");
  }
  else 
    Deleted=1;
  if (Deleted)
  {
    DelEnum(GetEnum(EnumName));
    ID=AddEnum(0, EnumName, 0);
  }
  return ID;
}

//
//  Function InsAddress2Enum
//    inserts address to enum
//    return 0 if successed,
//          error code otherwise
//
static InsAdd2Enum(Address, EnumName, Comment)
{
auto ID;
auto Result;
	ID=GetEnum(EnumName);
	if (ID == -1)
		ID=InitEnum(EnumName,1);
	Result=AddConstEx(ID, atoa(Address), Address, -1);
	if ( !Result )
	{
		GetConst(ID, Address, -1);
		SetConstCmt(GetConst(ID, Address, -1), Comment, 0);
	}
	return Result;
}

static Init ()
{
  InitEnum ( TYPEINFOENUM, 0 ) ;
  InitEnum ( RTTIENUM, 0 ) ;
}

// Enum RTTI
static Ins2RTTIEnum(Address)
{
  return InsAdd2Enum(Address, RTTIENUM , GetRTTIName(Address));
}

// Enum Type Info
static Ins2TIEnum(Address)
{
  return InsAdd2Enum(Address, TYPEINFOENUM, CopyPasStr(Address+1));
}





//
// TypeInfo
//

static TypeOrdComm(Address)
{
auto Ord;
auto Comment; 
	Ord=ReMakeByte(Address);
	if (Ord==otSByte)
		MakeComm(Address,"signed byte");
	if (Ord==otUByte)
		MakeComm(Address,"unsigned byte");
	if (Ord==otSWord)
		MakeComm(Address,"signed word");
	if (Ord==otUWord)
		MakeComm(Address,"unsigned word");
	if (Ord==otSLong)
		MakeComm(Address,"signed long");
}

#define RPKindGet	"Get"
#define RPKindSet	"Set"
#define RPKindStore	"Store"
#define RPKindIndex	"Index"

// hm, it`s not very clean function - just trust me...
static separatePointer(base_adr, adr, kind)
{
auto val;
	val = ReMakeDword(adr);
	if (0x80000000==val)
	{
		MakeComm(adr, kind+": none");
		return;
	}
	if (0xff000000==(0xff000000 & val)) // field index
	{
		val=val&0xffff;
		MakeComm(adr, kind+": offset 0x" + ltoa(val,0x10));
		return;
	}
	if (0xfe000000==(0xfe000000 & val)) // virtual function
	{
		val=val&0xffff;
		
		val=Dword(base_adr+76+val); // ptr in VTBL ??????????????????????????????????????????
		
		MakeComm(adr, kind+": virtual method at "+ltoa(val,0x10));
		return;
	}
	if (RPKindStore==kind && 1==val)
	{
		MakeComm(adr, kind+" always");
		return;
	}
	// otherwise - function
	MakeFuncOffset(adr);
	MakeComm(adr,kind+" function");
}

static MakePropList(Address, Count, TypeName, base_adr) 
{
auto Name;
auto Pointer;
auto TypePointer;
	while (Count)
	{
		Name="";
		Name=GetPasStr(Address+0x1A);
		Pointer=MakeOffset(Address);
		MakeComm(Address,"pointer to pointer to Type Info");
		
		//$$$MakeTypeInfo ( ReMakeDword ( Pointer ) ) ;
		//MakeTypeInfo(ReMakeDword(Pointer),ReMakeDword(Pointer)); // ???????????????????????????????????????
		ReMakeDword(Pointer);
		
		TypePointer=MakeOffset(Pointer);
		ReMakeByte(TypePointer);
		MakePasStr(TypePointer+1);
		MakeName(TypePointer,CopyPasStr(TypePointer+1));
		
		
		MakeName(Pointer, Name(Dword(Pointer))+"_Pointer");
		
		Address=Address+4;
		separatePointer(base_adr,Address, RPKindGet);
	//	MakeNamFuncOff(Address, TypeName+"@Get_"+Name);
	//	MakeComm(Address, "pointer to Get procedure");
		
		Address=Address+4;
		separatePointer(base_adr,Address, RPKindSet);
	//	MakeNamFuncOff(Address, TypeName+"@Set_"+Name);
	//	MakeComm(Address, "pointer to Set procedure");
		
		Address=Address+4;
		separatePointer(base_adr,Address, RPKindStore);
	//	MakeNamFuncOff(Address, TypeName+"@Stored_"+Name);
	//	MakeComm(Address, "pointer to Stored procedure");
		
		Address=Address+4;
		ReMakeDword(Address);
		MakeComm(Address, "Index");
		
		Address=Address+4;
		ReMakeDword(Address);
		MakeComm(Address, "Default");
		
		Address=Address+4;
		ReMakeWord(Address);
		MakeComm(Address, "NameIndex");
		
		Address=Address+2;
		MakeComm(Address, "Property name");
		Address=Address+Byte(Address)+1;
		Count--;
	}
}

static MakePropData(Address, TypeName, base_adr)
{
auto Count;
	Count=ReMakeWord(Address);
//Message("MakePropData - 1 "+ltoa(Count,0x10)+" \n");
	MakeComm(Address,ltoa(Count, 0x10)+" published properties");
	MakePropList(Address+2, Count, TypeName, base_adr);
}



static MakeTypeInfo(TypeInfoTable, Base_Adr)
{
auto Kind;
auto TTypeInfo;
auto PropAddress;
auto aType;
auto I;
auto Addr;
auto MinValue;
auto MaxValue;
auto StrAddress;
auto Count;
auto Pointer;
auto TypePointer;
	if (Ins2TIEnum(TypeInfoTable))
		return ;
	Kind=ReMakeByte(TypeInfoTable);
	MakeName(TypeInfoTable, GetPasStr(TypeInfoTable+1)+"_Type");
	MakeComm(TypeInfoTable+1, "name of type");
	TTypeInfo=GetAddrAfterStr(TypeInfoTable+1);
	if (Kind==tkUnknown)
	{
		MakeComm(TypeInfoTable,"kind of type : Unknow");
		return;
	}
	if (Kind==tkLString)
	{
		MakeComm(TypeInfoTable,"kind of type : LString");
		return;
	}
	if (Kind==tkWString)
	{
		MakeComm(TypeInfoTable, "kind of type : WString");
		return;
	}
	if (Kind==tkVariant)
	{
		MakeComm(TypeInfoTable,"kind of type : Variant");
		return;
	}
	if ((Kind == tkInteger)     ||
	    (Kind == tkChar)        || 
	    (Kind == tkEnumeration) ||
	    (Kind == tkWChar))
	{
		if (Kind==tkInteger)
			MakeComm(TypeInfoTable,"kind of type : Integer");
		if (Kind==tkChar)
			MakeComm(TypeInfoTable,"kind of type : Char");
		if (Kind==tkEnumeration )
			MakeComm(TypeInfoTable,"kind of type : Enumeration");
		if (Kind==tkWChar)
			MakeComm(TypeInfoTable,"kind of type : WChar");
		MinValue=ReMakeDword(TTypeInfo+1);
		MakeComm(TTypeInfo+1, "MinValue");
		MaxValue=ReMakeDword(TTypeInfo+5);
		MakeComm(TTypeInfo+5, "MaxValue");
		TypeOrdComm(TTypeInfo);
		if (Kind==tkEnumeration)
		{
			MakeOffset(TTypeInfo+0x9);
			MakeComm(TTypeInfo+0x9,"BaseType offset");
			//$$$MakeTypeInfo ( MakeOffset ( Dword ( TTypeInfo + 0x9 ) ) );
			MakeName(Dword(TTypeInfo),Name(Dword(Dword(TTypeInfo+0x9)))+"_Pointer");
			if (Dword(Dword(TTypeInfo+0x9))==TypeInfoTable)
			{
				MakeComm(TTypeInfo+0x0D,"NameList");
				StrAddress=TTypeInfo+0x0D;
				for (I=0;  I<=Abs(MaxValue-MinValue); I++)
				{
					MakePasStr(StrAddress);
					StrAddress=GetAddrAfterStr(StrAddress);
				}
			}
		}
		return ;
	}
	if (Kind==tkSet)
	{
		MakeComm(TypeInfoTable,"kind of type : Set");
		TypeOrdComm(TTypeInfo);
		MakeOffset(TTypeInfo+1);
		MakeComm(TTypeInfo+1,"CompType offset");
		return;
	}
	if (Kind==tkFloat)
	{
		MakeComm(TypeInfoTable,"kind of type : Float");
		aType=ReMakeByte(TTypeInfo);
		if (aType==ftSingle)
			MakeComm(TTypeInfo,"Single");
		if (aType==ftDouble)
			MakeComm(TTypeInfo,"Double");
		if (aType==ftExtended)
			MakeComm(TTypeInfo,"Extended");
		if (aType==ftComp)
			MakeComm(TTypeInfo,"Comp");
		if (aType==ftCurr)
			MakeComm(TTypeInfo,"Currency");
		return ;
	}
	if (Kind==tkString)
	{
		MakeComm(TTypeInfo,"kind of type : String");
		ReMakeByte(TTypeInfo);
		MakeComm(TTypeInfo,"MaxLength");
		return;
	}
	if (Kind==tkClass)
	{
		MakeComm(TypeInfoTable,"kind of type : Class");
		MakeOffset(TTypeInfo);
		MakeComm(TTypeInfo,"Class Type");
		Pointer= MakeOffset(TTypeInfo+4);
		//$$$MakeTypeInfo(ReMakeDword(Pointer));
		
		ReMakeDword(Pointer);

		TypePointer=MakeOffset(Pointer);
		ReMakeByte(TypePointer);
		MakePasStr(TypePointer+1);
		MakeName(TypePointer,CopyPasStr(TypePointer+1)+"_1");

		MakeName(Pointer,Name(Dword(Pointer))+"_Pointer"); //     
		MakeComm(TTypeInfo+4,"Parent Type Info");
		ReMakeWord(TTypeInfo+8);
		MakeComm(TTypeInfo+8,"Property Count");
		MakePasStr(TTypeInfo+0xA);
		MakeComm(TTypeInfo+0xA,"Unit name");
		MakePropData(GetAddrAfterStr(TTypeInfo+0xA),GetPasStr(TypeInfoTable+1), Base_Adr);
		return;
	}
	if (Kind==tkMethod)
	{
		MakeComm(TypeInfoTable,"kind of type : Method");
		TypeMethodKind(TTypeInfo);
		Count=ReMakeByte(TTypeInfo+1);
		MakeComm(TTypeInfo+1,"Parameters Count"); 
		Addr=TTypeInfo+2;
		for (I=0; I<Count; I++)
		{
			TypeParamFlag(Addr++);
			MakePasStr(Addr);
			MakeComm(Addr,"Parameter Name");
			Addr=GetAddrAfterStr(Addr);
			MakePasStr(Addr);
			MakeComm(Addr,"Type Name");
			Addr=GetAddrAfterStr(Addr);
		}
		if (Byte(TTypeInfo)==mkFunction)
		{
			MakePasStr(Addr);
			MakeComm(Addr,"Result Type");
		}
		return;
	}
	if (Kind==tkInterface)
	{
		MakeComm(TypeInfoTable,"kind of type : Interface");
		MakeOffset(TTypeInfo);
		MakeComm(TTypeInfo,"pointer to pointer to ancestor");
		//$$$MakeTypeInfo(MakeOffset(Dword(TTypeInfo)));
		MakeName(Dword(TTypeInfo ),
		       Name(Dword(Dword(TTypeInfo)))+"_Pointer");
		TypeIfaceFlag(TTypeInfo+4);
		TypeGUID(TTypeInfo+5);
		MakePasStr(TTypeInfo+0x15);
		MakeComm(TTypeInfo+0x15,"Interface Unit");
		PropAddress=GetAddrAfterStr(TTypeInfo+0x15);
		MakePropData(PropAddress,GetPasStr(TypeInfoTable+1), Base_Adr);
		return;
	}
	if (Kind==tkInt64)
	{
		MakeComm(TypeInfoTable,"kind of type : Int64");
		ReMakeQword(TTypeInfo);
		MakeComm(TTypeInfo,"MinInt64Value");
		ReMakeQword(TTypeInfo+8);
		MakeComm(TTypeInfo+8,"MaxInt64Value");
		return;
	}
	if (Kind==tkArray)
	{
		MakeComm(TypeInfoTable,"kind of type : Array");
		return;
	}
	if (Kind==tkRecord)
	{
		MakeComm(TypeInfoTable,"kind of type : Record");
		return;
	}
	if (Kind==tkDynArray)
	{
		MakeComm(TypeInfoTable,"kind of type : DynArray");
		return;
	}
}



//*************************************************************************************
//
// Functions for operation with RTTI
//
//*************************************************************************************

//
// Function GetRTTIName
//   get name of class of RTTI on start Address
// returns Str
//

static GetRTTIName(Address)
{
auto NamePointer;
	NamePointer=MakeOffset(Address+0x20);
	if (NamePointer!=0)
		return GetPasStr(NamePointer);
	else
		return "";
}

// 0x00
static MakeVTBL(Address)
{
  auto ClassName;
  auto Offset;
  auto count, my_name, vtbl_end;
  Offset=MakeOffset(Address);
  ClassName=GetRTTIName(Address);
  MakeName(Address, ClassName);
  MakeComm(Address, "RTTI+0x0 => -0x4C virtual method table");
  if (!Offset)
  	return;
  MakeName(Offset, ClassName + "_VTBL");
  vtbl_end=0;
  for(count=4; count<40; count=count+4)
  {
  	my_name=Dword(Address+count);
  	if (my_name>=Offset)
  	{
  		if (!vtbl_end)
  			vtbl_end=my_name;
  		else
  			if (vtbl_end>my_name)
  				vtbl_end=my_name;
  	}
  }
	my_name=0;
  for (count=vtbl_end-Offset; count>0; count=count-4)
  {
  	MakeFuncOffset(Offset);
	MakeComm(Offset,"VTBL+0x"+ltoa(my_name, 0x10));
//       !!!
	my_name=my_name+4;
  	Offset=Offset+4;
  }
}

// 0x04
////////////////////////////////////////////////////////////////////////************************************************
static padZeros(str,desired_len)
{
  auto len;
  len = strlen(str);
  len = desired_len - len;
  while ( len )
  {
    str = "0" + str;
    len--;
  }
 return str;
}

// makes TGUID
static make_TGUID(adr)
{
  auto count;
 // first DWORD
  ReMakeDword(adr);
  adr = adr + 4;
 // next - two WORD`s
  ReMakeWord(adr);
  ReMakeWord(adr+2);
  adr = adr + 4;
  for ( count = 0; count < 8; count++ )
   ReMakeByte(adr++);
}

static getTGUIDstr(adr)
{
  auto res, count;

  count = Dword(adr);
  res = "{" + padZeros( ltoa(count,0x10), 8 );
  adr = adr + 4;
  count = Word(adr);
  res = res + "-" + padZeros( ltoa(count,0x10), 4);
  adr = adr + 2;
  count = Word(adr);
  res = res + "-" + padZeros( ltoa(count,0x10), 4);
  adr = adr + 2;
  count = Byte(adr++);
  res = res + "-" + padZeros( ltoa(count,0x10), 2);
  count = Byte(adr++);
  res = res + padZeros( ltoa(count,0x10), 2) + "-";
  for ( count = 0; count < 6; count++ )
    res = res + padZeros( ltoa(Byte(adr+count),0x10), 2);
  return res + "}";
}

///////////////////////////////////////////////////////////////////////*************************************************
static MakeIntrfaceTbl(Address)
{
  auto Offset;
  auto CurrentAddress;
  auto Count;
  auto I;
//  auto EntrySize;
  CurrentAddress=Address+0x4;
  Offset=MakeOffset(CurrentAddress);
  MakeComm(CurrentAddress, "-0x48 pointer to interface table");
  if (!Offset)
    return;
  MakeName(Dword(CurrentAddress),GetRTTIName(Address)+"_Interface_Table");
  CurrentAddress=Dword(CurrentAddress);
  Count=ReMakeDword(CurrentAddress);
  MakeComm(CurrentAddress,"Interfaces Count");
  CurrentAddress=CurrentAddress+4;
  for(I=0; I<Count; I++)
  {
    // TGUID
    make_TGUID(CurrentAddress);
    MakeComm(CurrentAddress, getTGUIDstr(CurrentAddress));
    CurrentAddress = CurrentAddress + 0x10;
    // next - pointer to VTBL
    MakeOffset(CurrentAddress);
    MakeComm(CurrentAddress, "pointer to VTBL" );
    CurrentAddress = CurrentAddress + 4;
    // next - IOffset
    MakeComm(CurrentAddress, "Offset " + ltoa(MakeOffset(CurrentAddress), 0x10));
    CurrentAddress = CurrentAddress + 4;
   // next - ImplGetter
    ReMakeDword(CurrentAddress);
    CurrentAddress = CurrentAddress + 4;
  }
}



// 0x08
static MakeOLETbl ( Address )
{
  auto Offset ;
  auto CurrentAddress ;
  auto I ;
  auto Count ;
  CurrentAddress = Address + 0x8 ;
  Offset = MakeOffset ( CurrentAddress ) ;
  MakeComm ( CurrentAddress, "-0x44 pointer to Automation information table") ;
  if ( !Offset )
    return ;
  MakeName ( Dword ( CurrentAddress ), GetRTTIName ( Address ) + "_Automation_Table" ) ;
  CurrentAddress = Dword ( CurrentAddress ) ;
  Count = ReMakeDword ( CurrentAddress ) ;
  MakeComm ( CurrentAddress, "Automation entry count" ) ;
  CurrentAddress = CurrentAddress + 4 ;
  for ( I = 0; I < Count; I++ )
  {
    ;MakeAutoEntry ( CurrentAddress + I * 0x14 ) ;
  }
}

// 0x0C
static MakeInstanceTbl ( Address )
{
  auto Offset ;
  auto CurrentAddress ;
  CurrentAddress = Address + 0x0C ;
  Offset = MakeOffset ( CurrentAddress ) ;
  MakeComm ( CurrentAddress, "-0x40 pointer to instance initialization table") ;
  if ( !Offset )
    return ;
  MakeName ( Dword ( CurrentAddress ), GetRTTIName ( Address ) + "_Instance_Init_Table" ) ;
}

// 0x10
static MakeTypeInfoTbl(Address)
{
auto Offset;
auto CurrentAddress;
	CurrentAddress=Address+0x10;
	Offset=MakeOffset(CurrentAddress);
	MakeComm(CurrentAddress, "-0x3C pointer to type information table");
	if (!Offset)
		return;
//	MakeTypeInfo(Dword(CurrentAddress));
	MakeTypeInfo(Offset, Address);
}

// 0x14
static MakeFieldDefTbl(Address)
{
auto vName, field_name;
auto Offset;
auto CurrentAddress;
auto I, ID;
auto Count;
auto FieldTypes;
auto StrOffset, StrOffset1;
auto FieldPointer;
	CurrentAddress=Address+0x14;
	Offset=MakeOffset(CurrentAddress);
	MakeComm(CurrentAddress,"-0x38 pointer to field definition table");
	if (!Offset)
		return ;
	vName=GetRTTIName(Address);
	ID=InitEnum(vName+"_field",0);
//	MakeName(Dword(CurrentAddress),GetRTTIName(Address)+"_Field_Definifion_Table");
	MakeName(Dword(CurrentAddress),vName+"_Field_Definifion_Table");
	CurrentAddress= Dword(CurrentAddress);
	Count=ReMakeWord(CurrentAddress);
	MakeComm(CurrentAddress,"Fiels count");
	CurrentAddress=CurrentAddress+2;
	FieldTypes=MakeOffset(CurrentAddress);
	MakeName(FieldTypes,GetRTTIName(Address)+"_Field_Types_Array");
	CurrentAddress=CurrentAddress+4;
	for (I=0; I<Count; I++)
	{
		StrOffset1=ReMakeDword(CurrentAddress);
		MakeComm(CurrentAddress,"offset 0x"+ltoa(StrOffset, 0x10)); // ********
		CurrentAddress=CurrentAddress+4;
		StrOffset=ReMakeWord(CurrentAddress);
		MakeComm(CurrentAddress,"Type: "+GetRTTIName(Dword(FieldTypes+2+StrOffset*4)));
		CurrentAddress=CurrentAddress+2;
		MakePasStr(CurrentAddress);
		field_name=GetPasStr(CurrentAddress);
		CurrentAddress=GetAddrAfterStr(CurrentAddress);
//         
		AddConstEx(ID,vName+"_"+field_name,StrOffset1,-1);
Message(vName+"_"+field_name+"    "+ltoa(StrOffset,0x10)+" \n");
	}
	Count=ReMakeWord(FieldTypes);
	CurrentAddress=FieldTypes+2;
	for (I=0;I<Count; I++)
	{
		FieldPointer=MakeOffset(CurrentAddress+I*4);
		ReMakeDword(FieldPointer);
		//MakeRTTI(MakeOffset(CurrentAddress+I*4));
	}
}

// 0x18
static MakeMthdDefTbl ( Address )
{
  auto Offset ;
  auto CurrentAddress ;
  auto I , Count ;
  auto MethodAddress ;
  CurrentAddress = Address + 0x18;
  Offset = MakeOffset ( CurrentAddress ) ;
  MakeComm ( CurrentAddress, "-0x34 pointer to method definition table") ;
  if ( !Offset )
    return ;
  MakeName ( Offset, GetRTTIName ( Address ) + "_Method_Definifion_Table" ) ;
  CurrentAddress = Offset ; 
  Count = ReMakeWord ( CurrentAddress ) ;
  MakeComm ( CurrentAddress, "Methods count" ) ;
  CurrentAddress = CurrentAddress + 2 ;
  for ( I = 0; I < Count; I++ )
  {
    ReMakeWord ( CurrentAddress ) ;
    MakeComm ( CurrentAddress, "type of method" ) ;
    CurrentAddress = CurrentAddress + 2 ;
    MethodAddress = CurrentAddress ;
    CurrentAddress = CurrentAddress + 4 ;
    MakePasStr ( CurrentAddress ) ;
    MakeNamFuncOff ( MethodAddress ,
               GetRTTIName ( Address ) + "_" + CopyPasStr ( CurrentAddress ));
    CurrentAddress = GetAddrAfterStr ( CurrentAddress ) ;
  }
}

// 0x1C
static MakeDynMthdTbl ( Address )
{
  auto Offset ;
  auto CurrentAddress ;
  auto I, Count ;
  CurrentAddress = Address + 0x1C;
  Offset = MakeOffset ( CurrentAddress ) ;
  MakeComm ( CurrentAddress, "-0x30 pointer to dynamic method table") ;
  if ( !Offset )
   return ;
  CurrentAddress = Dword ( CurrentAddress ) ;
  MakeName ( CurrentAddress, GetRTTIName ( Address ) + "_Dynamic_Method_Table" ) ;
  Count = ReMakeWord ( CurrentAddress ) ;
  MakeComm ( CurrentAddress , "dynamic method count" ) ;
  CurrentAddress = CurrentAddress + 2 ; 
  for ( I=0; I < Count; I++ )
  {
    ReMakeWord ( CurrentAddress + I * 2 ) ;
    MakeComm ( CurrentAddress + I * 2, "dynamic method index") ;
  }
  CurrentAddress = CurrentAddress + Count * 2 ;
  for ( I=0; I < Count; I++ )
  {
    MakeFuncOffset ( CurrentAddress + I * 4 ) ;
  }
}

// 0x20
static MakeClassName (Address)
{
  auto ClassName;
  MakePasStr(ReMakeDword(Address+0x20));
  ClassName=GetRTTIName(Address);
  MakeComm(Address+0x20,"-0x2C "+ClassName+"_class name offset");
Message("Create class - " + ClassName + "\n");
}

// 0x24
static MakeSize ( Address )
{ 
  auto CurrentAddress ;
  CurrentAddress = Address + 0x24 ;
  ReMakeDword ( CurrentAddress ) ;
  MakeComm (CurrentAddress , "-0x28 instance size in bytes" ) ;
}

// 0x28
static MakeP2PParent ( Address )
{
  auto Offset ;
  auto CurrentAddress ;
//
  auto ClassName;
  ClassName=GetRTTIName(Address);
//
  CurrentAddress=Address + 0x28;
  Offset=MakeOffset (CurrentAddress);
  MakeOffset(Offset);
  MakeComm(CurrentAddress, "-0x24 " + ClassName + " to a pointer to ancestor class") ;
  return Offset ;
}

// 0x2C
static MakeIntrnlMthds(Address)
{
  auto ClassName;
  ClassName=GetRTTIName(Address);
  MakeFuncOffset(Address+0x2C);
  MakeComm(Address+0x2C,"-0x20 SafeCallException Method");
  MakeFuncOffset(Address+0x30);
  MakeComm(Address+0x30,"-0x1C AfterConstruction Method");
  MakeFuncOffset(Address+0x34);
  MakeComm(Address+0x34,"-0x18 BeforeConstruction Method");
  MakeFuncOffset(Address+0x38);
  MakeComm(Address+0x38,"-0x14 Dispatch Method");
  MakeFuncOffset(Address+0x3C);
  MakeComm(Address+0x3C,"-0x10 DefaultHandler Method");
  MakeFuncOffset(Address+0x40);
  MakeComm(Address+0x40,"-0x0C NewInstance Method");
  MakeFuncOffset(Address+0x44);
  MakeComm(Address+0x44,"-0x08 FreeInstance Method");
  MakeFuncOffset(Address+0x48);
  MakeComm(Address+0x48,"-0x04 Destroy Method");
}

//
//
//

// pre-check RTTI and ask about it
static askRTTI(adr)
{
auto name, dyn_count, what_say;
auto get_dyncount;
	get_dyncount=Dword(adr+0x1c);
	if (!get_dyncount)
		dyn_count=0;
	else
		dyn_count=Word(get_dyncount);
	name = GetRTTIName(adr);
	what_say = "Do you want to process RTTI of class \"" + name + "\"";
	if ( dyn_count )
		what_say = what_say + " with " + ltoa(dyn_count, 10) + " dynamic methods";
	what_say = what_say + "?";
	return AskYN(1, what_say);
}

static MakeRTTI(Address)
{
auto Parent;
	Parent=MakeP2PParent(Address);// 0x28 make a pointer to ancestor class (or nil)
	MakeClassName(Address);     	// 0x20 make Str containing class name
	if (Ins2RTTIEnum(Address))     //trying insert Address in RTTIENUM
		return Parent;                 //return if Address exist if RTTIENUM
	MakeName(Address, GetRTTIName(Address)+"_RTTI");
	MakeSize(Address);          	// 0x24 make instance size in bytes
	MakeVTBL(Address);            // 0x00 make virtual method table (or nil)
	MakeIntrfaceTbl(Address);     // 0x04 make interface table (or nil)
	MakeOLETbl(Address);        	// 0x08 make Automation information table (or nil)
	MakeInstanceTbl(Address);   	// 0x0Cmake instance initialization table (or nil)
	MakeFieldDefTbl(Address);   	// 0x14 make field definition table (or nil)
	MakeMthdDefTbl(Address);    	// 0x18 make method definition table (or nil)
	MakeDynMthdTbl(Address);      // 0x1C make dynamic method table (or nil)
	MakeIntrnlMthds(Address);   	// 0x2C make entry points of internal methods
	MakeTypeInfoTbl(Address); 	// 0x10 make type information table (or nil)
	// MakeUserMthds(Address);     //make entry points of user-defined virtual methods
	return Parent;
}


// ********************************************************************************************
//   Initialize, Finalize
//
static UnitEntries(adr)
{
	auto count, i;
	
	count = ReMakeDword(adr);
	MakeName(adr,"UnitEntries");
	MakeComm(adr,"Unit Entries");
	adr = MakeOffset(adr + 4);
	if ( !adr || ! count )
		return;
	for ( i = 0; i < count; i++ )
	{
		MakeFuncOffset(adr);
		MakeComm(adr,"Initialize");
		adr = adr + 4;
		MakeFuncOffset(adr);
		MakeComm(adr,"Finalize");
		adr = adr + 4;
	}
}

//****************************************************************************************
//*  
//****************************************************************************************

static IprocessRTTI()
{
auto Address;
	Address=ScreenEA();
	if (1==askRTTI(Address))
	{
		MakeRTTI(Address);	
	}
}

static IunitEntries()
{
	UnitEntries(ScreenEA());
}
