// IDA.idc
//
//   Delphi
//
// 13.05.2005

#include <idc.idc>
#include <d:\Programs\IDAidc\Delphi7.idc>

//*************************************************************************************
//
//  Functions for operation with byte, word and dword
//
//*************************************************************************************

//
// Function ReMakeByte
//   makes byte on address Address
//   return value of byte on address Address
//
static ReMakeByte ( Address )
{
  MakeUnkn ( Address, 0 );
  MakeByte ( Address );
  return Byte ( Address );
}

//
// Function ReMakeWord
//   makes word on address Address
//   return value of word on address Address
//
static ReMakeWord ( Address )
{
  MakeUnkn ( Address, 0 );
  MakeUnkn ( Address + 1, 0 );
  MakeWord ( Address );
  return Word ( Address );
}

//
// Function ReMakeDword
//   makes dword on address Address
//   return value of dword on address Address
//
static ReMakeDword ( Address )
{
  MakeUnkn ( Address, 0 );
  MakeUnkn ( Address + 1, 0 );
  MakeUnkn ( Address + 2, 0 );
  MakeUnkn ( Address + 3, 0 );
  MakeDword ( Address );
  return Dword ( Address );
}

//
// Function ReMakeQword
//   makes dword on address Address
//   return value of dword on address Address
//
static ReMakeQword ( Address )
{
  MakeUnkn ( Address, 0 );
  MakeUnkn ( Address + 1, 0 );
  MakeUnkn ( Address + 2, 0 );
  MakeUnkn ( Address + 3, 0 );
  MakeUnkn ( Address + 4, 0 );
  MakeUnkn ( Address + 5, 0 );
  MakeUnkn ( Address + 6, 0 );
  MakeUnkn ( Address + 7, 0 );
  MakeQword ( Address );
  return ; // Qword ( Address );
}

//*************************************************************************************
//
//  Functions for operation with Str
//
//
//*************************************************************************************

//
//  Function ReMakeStr
//    makes Str to data on Address with length equal Length
//   return 1 if success
//          0 if failure
static ReMakeStr ( Address, Length )
{
  auto I;
  for ( I = 0; I < Length; I++ )
    MakeUnkn ( Address + I, 0 );
  return MakeStr( Address , Address + Length );
}

//
//  Function MakePasStr
//    makes Pas-style Str
//   return 1 if success
//          0 if failure
static MakePasStr(Address)
{
auto Length;
	Length=ReMakeByte(Address);
	return ReMakeStr(Address,Length+1);
}

//
//  Function MakeCurPasStr
//    makes Pas-style Str
//   return 1 if success
//          0 if failure
static MakeCurPasStr ()
{
  return MakePasStr( ScreenEA () );
}

//
//  Function GetPasStr
//    get Pas-style Str
//  return Str
//
static GetPasStr( Address )
{
  auto Length, Result;
  MakePasStr ( Address ) ;
  Length = Byte ( Address++ );
  Result = "";
  for ( ; Length; Length-- )
    Result = Result + Byte( Address++ );
  return Result;
}

static GetZStr( Address )
{
  auto Length, Result;
  MakeStr ( Address,BADADDR ) ;
  Result = "";
  while (Byte(Address))
    Result = Result + Byte( Address++ );
  return Result;
}

//
//  Function CopyPasStr
//    get Pas-style Str
//  return Str
//
static CopyPasStr( Address )
{
  auto Length, Result;
  Length = Byte ( Address++ );
  Result = "";
  for ( ; Length; Length-- )
    Result = Result + Byte( Address++ );
  return Result;
}

//
//  Function GetPasStr
//    get address after end of Pas-style Str
//  return Address
//
static GetAddrAfterStr(Address)
{
	Address=Address+Byte(Address)+1;
	return Address ;
}

//*************************************************************************************
//
//  Function for operation with offset
//
//*************************************************************************************

//
// Function MakeOffset
//   makes offset to data on Address
//   return offset to data
//

static MakeOffset ( Address )
{
  auto Offset;
  Offset = ReMakeDword ( Address );
  if ( Offset != 0 )
    OpOff( Address, 0, 0 );
  return Offset ;
}

//
// Function MakeFuncOffset
//   makes dword and offset to a function
//   return 1 if success
//          0 if failure
static MakeFuncOffset ( Address )
{
  auto FuncOffset ;
  FuncOffset = MakeOffset ( Address ) ;
  if ( FuncOffset == 0 )
    return 0 ;
  MakeUnkn ( FuncOffset, 1 ) ;
  if ( MakeFunction ( FuncOffset, BADADDR ) == 0 )
    return 0 ;
  add_dref ( Address, FuncOffset, 0 );
  return 1 ;
}

// Function MakeNamFuncOff
//   makes named function on Address
//   return 1 if success
//          0 if failure
static MakeNamFuncOff ( Address, Name )
{
  if ( !MakeFuncOffset ( Address ) )
    return 0 ;
  MakeName ( Dword ( Address ), Name ) ;
  return 1 ;
}

// ********************************************************************************************
//
//
// ********************************************************************************************
//   Initialize, Finalize
//
static UnitEntries(adr)
{
	auto count, i;
	
	count = ReMakeDword(adr);
	MakeName(adr,"PackageInfoTable");
	MakeComm(adr,"number of entries in UnitInfo array; always > 0");
	adr = MakeOffset(adr + 4);
	if ( !adr || ! count )
		return;
	MakeName(adr,"UnitEntryTable");
	for ( i = 0; i < count; i++ )
	{
		MakeFuncOffset(adr);
		MakeComm(adr,"Initialize");
		adr = adr + 4;
		MakeFuncOffset(adr);
		MakeComm(adr,"Finalize");
		adr = adr + 4;
	}
}
//   Init table - InitImports
//
static InitTable(adr)
{
	auto count, i;
	
	count = ReMakeDword(adr);
//	MakeName(adr,"PackageInfoTable");
	MakeComm(adr,"number of init table");
	if (!count)
		return;
	adr=adr+4;
//	MakeName(adr,"UnitEntryTable");
	for ( i = 0; i < count; i++ )
	{
		MakeOffset(adr);
		MakeComm(adr,"variableAddress");
		adr = adr + 4;
		MakeOffset(adr);
		MakeComm(adr,"sourceAddress");
		adr = adr + 4;
		MakeOffset(adr);
		MakeComm(adr,"sourceOffset");
		adr = adr + 4;
	}
}

// ********************************************************************************************
//
//
// ********************************************************************************************
//   :
// 

static Struct_System()
{
auto	sHandle;
auto	error;
auto	STRUCTURE_NAME;
	STRUCTURE_NAME = "TInitContext_obj";
	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{
		sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{
			WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
		}
	}
	SetStrucComment(sHandle,"Module initialization context.  For internal use only.",1);
	AddStrucMember(sHandle, "OutherContext",	0x00, FF_DWRD | FF_0OFF, -1, 4);
	SetMemberComment(sHandle,0x00,"saved InitContext",1);
	AddStrucMember(sHandle, "ExcFrame",		0x04, FF_DWRD | FF_0OFF, -1, 4);
	SetMemberComment(sHandle,0x04,"bottom exc handler",1);
	AddStrucMember(sHandle, "InitTable",		0x08, FF_DWRD | FF_0OFF, -1, 4);
	SetMemberComment(sHandle,0x08,"unit init info",1);
	AddStrucMember(sHandle, "InitCount",		0x0C, FF_DWRD, -1, 4);
	SetMemberComment(sHandle,0x0C,"how far we got",1);
	AddStrucMember(sHandle, "Module",		0x10, FF_DWRD | FF_0OFF, -1, 4);
	SetMemberComment(sHandle,0x10,"ptr to module desc",1);
	AddStrucMember(sHandle, "DLLSaveEBP",		0x14, FF_DWRD | FF_0OFF, -1, 4);
	SetMemberComment(sHandle,0x14,"saved regs for DLLs",1);
	AddStrucMember(sHandle, "DLLSaveEBX",		0x18, FF_DWRD | FF_0OFF, -1, 4);
	SetMemberComment(sHandle,0x18,"saved regs for DLLs",1);
	AddStrucMember(sHandle, "DLLSaveESI",		0x1C, FF_DWRD | FF_0OFF, -1, 4);
	SetMemberComment(sHandle,0x1C,"saved regs for DLLs",1);
	AddStrucMember(sHandle, "DLLSaveEDI",		0x20, FF_DWRD | FF_0OFF, -1, 4);
	SetMemberComment(sHandle,0x20,"saved regs for DLLs",1);
	AddStrucMember(sHandle, "ExitProcessTLS",	0x24, FF_DWRD | FF_0OFF, -1, 4);
	SetMemberComment(sHandle,0x24,"Shutdown for TLS",1);
	AddStrucMember(sHandle, "DLLInitState",         0x28, FF_BYTE, -1, 1); 
	SetMemberComment(sHandle,0x28,"0 = package, 1 = DLL shutdown, 2 = DLL startup",1);
	// 
	STRUCTURE_NAME = "TLibModule_obj";
	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{
		sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{
			WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
		}
	}
	SetStrucComment(sHandle,"Package/Module registration and unregistration",1);
	AddStrucMember(sHandle, "Next",			0x00, FF_DWRD | FF_0OFF, -1, 4);
	AddStrucMember(sHandle, "Instance",		0x04, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "CodeInstance",		0x08, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "DataInstance",		0x0C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "ResInstance",		0x10, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "Reserved",		0x14, FF_WORD, -1, 2);
}

// ********************************************************************************************
//
//
// ********************************************************************************************
//   :
// TObject, TPersistent

static Struct_TObject()
{
auto	sHandle;
auto	error;
auto	STRUCTURE_NAME;

auto ID, Address;
auto EnumName;
	EnumName="VMT";
	ID=AddEnum(0, EnumName, 0);
	if (ID != -1)
	{	Address=-0x4C; AddConstEx(ID, "vmtSelfPrt", Address, -1);
		Address=-0x48; AddConstEx(ID, "vmtIntfTable", Address, -1);
		Address=-0x44; AddConstEx(ID, "vmtAutoTable", Address, -1);
		Address=-0x40; AddConstEx(ID, "vmtInitTable", Address, -1);
		Address=-0x3C; AddConstEx(ID, "vmtTypeInfo", Address, -1);
		Address=-0x38; AddConstEx(ID, "vmtFieldTable", Address, -1);
		Address=-0x34; AddConstEx(ID, "vmtMethodTable", Address, -1);
		Address=-0x30; AddConstEx(ID, "vmtDynamicTable", Address, -1);
		Address=-0x2C; AddConstEx(ID, "vmtClassName", Address, -1);
		Address=-0x28; AddConstEx(ID, "vmtInstanceSize", Address, -1);
		Address=-0x24; AddConstEx(ID, "vmtParent", Address, -1);
		Address=-0x20; AddConstEx(ID, "vmtSafeCallException", Address, -1);
		Address=-0x1C; AddConstEx(ID, "vmtAfterConstruction", Address, -1);
		Address=-0x18; AddConstEx(ID, "vmtBeforeDestruction", Address, -1);
		Address=-0x14; AddConstEx(ID, "vmtDispatch", Address, -1);
		Address=-0x10; AddConstEx(ID, "vmtDefaultHandler", Address, -1);
		Address=-0x0C; AddConstEx(ID, "vmtNewInstance", Address, -1);
		Address=-0x08; AddConstEx(ID, "vmtFreeInstance", Address, -1);
		Address=-0x04; AddConstEx(ID, "vmtDestroy", Address, -1);
		Address= 0x00; AddConstEx(ID, "vmtQueryInterface", Address, -1);
		Address= 0x04; AddConstEx(ID, "vmtAddRef", Address, -1);
		Address= 0x08; AddConstEx(ID, "vmtRelease", Address, -1);
		Address= 0x0C; AddConstEx(ID, "vmtCreateObject", Address, -1);	}

// File mode magic numbers 
	EnumName="FileModeMagicNumbers";
	ID=AddEnum(0, EnumName, 0);
	if (ID != -1)
	{	Address=0xD7B0; AddConstEx(ID, "fmClosed", Address, -1);
		Address=0xD7B1; AddConstEx(ID, "fmInput", Address, -1);
		Address=0xD7B2; AddConstEx(ID, "fmOutput", Address, -1);
		Address=0xD7B3; AddConstEx(ID, "fmInOut", Address, -1);	}

	STRUCTURE_NAME = "TObject_obj";
	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{	sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{	WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
			return -1;	}
		return sHandle;	}
	AddStrucMember(sHandle, "pVTBL",		0x00, FF_DWRD, -1, 4);
//	AddStrucMember(sHandle, "pVTBL",		0x00, FF_ASCI, -1, 2);
//	AddStrucMember(sHandle, "overlay_number",      0x1a, FF_WORD, -1, 2); 
//	AddStrucMember(sHandle, "reserved",            0x1c, FF_BYTE, -1, 32); 
//	AddStrucMember(sHandle, "new_hdr_offset",      0x3c, FF_DWRD, -1, 4);
	return sHandle;
}

static Struct_TClassFinder()
{
auto	sHandle, sHandle_TObject;
auto	error;
auto	NAME, STRUCTURE_NAME, STRUCTURE_PARENT;
auto ID, Address;
auto EnumName;

// Init Data
	NAME = "TClassFinder";
	STRUCTURE_PARENT = "TObject_obj";
	STRUCTURE_NAME = NAME+"_obj";
	EnumName="Enum"+NAME;

	//     ,    
	sHandle_TObject = GetStrucIdByName(STRUCTURE_PARENT);
	if(sHandle_TObject == -1)
	{	sHandle_TObject = Struct_TObject();	} // ************** !!!!!!!!!!!!!!!

	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{	sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{	WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
			return -1;	}
		return sHandle;		}
		
	AddStrucMember(sHandle, "pVTBL",		0x00, FF_DWRD, -1, 0x4);
	AddStrucMember(sHandle, "FGroups",		0x04, FF_DWRD, -1, 0x4);
}


static Struct_TList()
{
auto	sHandle, sHandle_TObject;
auto	error;
auto	STRUCTURE_NAME;

auto ID, Address;
auto EnumName;
	//     TObject,    
	sHandle_TObject = GetStrucIdByName("TObject_obj");
	if(sHandle_TObject == -1)
	{	sHandle_TObject = Struct_TObject();	}

	STRUCTURE_NAME = "TList_obj";
	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{	sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{	WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
			return -1;	}
		return sHandle;		}
		
//	AddStrucMember(sHandle, "pVTBL",		0x00, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "pVTBL",		0x00, FF_DWRD + FF_0OFF, -1, 4);
	AddStrucMember(sHandle, "Items",		0x04, FF_DWRD + FF_0OFF, -1, 4);
	AddStrucMember(sHandle, "Count",		0x08, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "Capacity",		0x0C, FF_DWRD, -1, 4);
	
	EnumName="EnumTList";
	ID=AddEnum(0, EnumName, 0);
	if (ID != -1)
	{	Address=0x00; AddConstEx(ID, "TList_Grow", Address, -1);
		Address=0x04; AddConstEx(ID, "TList_4", Address, -1);
		Address=0x08; AddConstEx(ID, "TList_8", Address, -1);
		Address=0x0C; AddConstEx(ID, "TList_10", Address, -1);	}

	return sHandle;
}


static Struct_TPersistent()
{
auto	sHandle, sHandle_TObject;
auto	error;
auto	STRUCTURE_NAME;
auto ID, Address;
auto EnumName;
	//     TObject,    
	sHandle_TObject = GetStrucIdByName("TObject_obj");
	if(sHandle_TObject == -1)
	{	sHandle_TObject = Struct_TObject();	}

	STRUCTURE_NAME = "TPersistent_obj";
	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{	sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{	WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
			return -1;	}
		return sHandle;		}
		
	AddStrucMember(sHandle, "pVTBL",		0x00, FF_DWRD + FF_0OFF, -1, 4);
	EnumName="EnumTPersistent";
	ID=AddEnum(0, EnumName, 0);
	if (ID != -1)
	{	Address=0x00; AddConstEx(ID, "TPersistent_0", Address, -1);
		Address=0x04; AddConstEx(ID, "TPersistent_4", Address, -1);
		Address=0x08; AddConstEx(ID, "TPersistent_8", Address, -1);	}
	return sHandle;
}

static Struct_TComponent()
{
auto	sHandle;
auto	error;
auto	STRUCTURE_NAME;
auto ID, Address;
auto EnumName;
auto	sHandle_Obj;
	EnumName="TComponentState";
	ID=AddEnum(0, EnumName, 0);
	if (ID != -1)
	{	Address=0x01; AddConstEx(ID, "csLoading", Address, -1);
		Address=0x02; AddConstEx(ID, "csReading", Address, -1);
		Address=0x03; AddConstEx(ID, "csWriting", Address, -1);
		Address=0x04; AddConstEx(ID, "csDestroying", Address, -1);
		Address=0x05; AddConstEx(ID, "csDesigning", Address, -1);
		Address=0x06; AddConstEx(ID, "csAncestor", Address, -1);
		Address=0x07; AddConstEx(ID, "csUpdating", Address, -1);
		Address=0x08; AddConstEx(ID, "csFixups", Address, -1);
		Address=0x09; AddConstEx(ID, "csFreeNotification", Address, -1);
		Address=0x0a; AddConstEx(ID, "csInline", Address, -1);
		Address=0x0b; AddConstEx(ID, "csDesignInstance", Address, -1);
	}
	//     TPersestent,    
	sHandle_Obj = GetStrucIdByName("TRersistent_obj");
	if(sHandle_Obj == -1)
	{	sHandle_Obj = Struct_TPersistent();	}


	STRUCTURE_NAME = "TComponent_obj";
	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{	sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{	WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
			return -1;	}
		return sHandle;		}
	AddStrucMember(sHandle, "TPersistent",		0x00, FF_STRU, sHandle_Obj, 4);
	AddStrucMember(sHandle, "FOwner",		0x04, FF_DWRD + FF_0OFF, -1, 4); // FOwner: TComponent;
	AddStrucMember(sHandle, "FName",		0x08, FF_DWRD + FF_0OFF, -1, 4); // TComponentName; 
	AddStrucMember(sHandle, "TComponentC",		0x0C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "FComponents",		0x10, FF_DWRD + FF_0OFF, -1, 4);
	AddStrucMember(sHandle, "TComponent14",		0x14, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "FRoot",		0x18, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "ComponentState",	0x1C, FF_BYTE, -1, 1);
	AddStrucMember(sHandle, "TComponent20",		0x20, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "FComponentStyle",	0x24, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TComponent28",		0x28, FF_DWRD + FF_0OFF, -1, 4);
	AddStrucMember(sHandle, "TComponent2C",		0x2C, FF_DWRD + FF_0OFF, -1, 4);
	EnumName="EnumTComponent";
	ID=AddEnum(0, EnumName, 0);
	if (ID != -1)
	{	Address=0x00; AddConstEx(ID, "TComponent_0", Address, -1);
		Address=0x04; AddConstEx(ID, "TComponent_4", Address, -1);
		Address=0x08; AddConstEx(ID, "TComponent_8", Address, -1);
		Address=0x0C; AddConstEx(ID, "TComponent_Loaded", Address, -1);
		Address=0x10; AddConstEx(ID, "TComponent_10", Address, -1);
		Address=0x14; AddConstEx(ID, "TComponent_14", Address, -1);
		Address=0x18; AddConstEx(ID, "TComponent_SetName", Address, -1);
		Address=0x1C; AddConstEx(ID, "TComponent_FComponentState", Address, -1);
		Address=0x20; AddConstEx(ID, "TComponent.ValidateRename", Address, -1);
		Address=0x24; AddConstEx(ID, "TComponent_24", Address, -1);
		Address=0x28; AddConstEx(ID, "TComponent_28", Address, -1);
		Address=0x2C; AddConstEx(ID, "TComponent.Create", Address, -1);	}
	return sHandle;
}

static Struct_TTimer()
{
auto	sHandle;
auto	error;
auto	STRUCTURE_NAME;
auto ID, Address;
auto EnumName;
	//     TPersestent,    
auto	sHandle_Obj;
	sHandle_Obj = GetStrucIdByName("TComponent_obj");
	if(sHandle_Obj == -1)
	{	sHandle_Obj = Struct_TComponent();	}


	STRUCTURE_NAME = "TTimer_obj";
	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{	sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{	WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
			return -1;	}
		return sHandle;		}
	AddStrucMember(sHandle, "TComponent",		0x00, FF_STRU, sHandle_Obj, 0x30);
	AddStrucMember(sHandle, "FInterval",		0x30, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "FWindowHandle",	0x34, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "FOnTimer",		0x38, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TTimer3C",		0x3C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "FEnabled",		0x40, FF_BYTE, -1, 1);
	AddStrucMember(sHandle, "TTimer41",		0x41, FF_BYTE, -1, 1); 
	AddStrucMember(sHandle, "TTimer42",		0x42, FF_BYTE, -1, 1); 
	AddStrucMember(sHandle, "TTimer43",		0x43, FF_BYTE, -1, 1); 
	EnumName="EnumTComponent";
	ID=AddEnum(0, EnumName, 0);
	if (ID != -1)
	{	Address=0x00; AddConstEx(ID, "TTimer_0", Address, -1);
		Address=0x04; AddConstEx(ID, "TTimer_4", Address, -1);
		Address=0x08; AddConstEx(ID, "TTimer_8", Address, -1);
		Address=0x0C; AddConstEx(ID, "TTimer_C", Address, -1);
		Address=0x10; AddConstEx(ID, "TTimer_10", Address, -1);
		Address=0x14; AddConstEx(ID, "TTimer_14", Address, -1);
		Address=0x18; AddConstEx(ID, "TTimer_18", Address, -1);
		Address=0x1C; AddConstEx(ID, "TTimer_1C", Address, -1);
		Address=0x20; AddConstEx(ID, "TTimer_20", Address, -1);
		Address=0x24; AddConstEx(ID, "TTimer_24", Address, -1);
		Address=0x28; AddConstEx(ID, "TTimer_28", Address, -1);
		Address=0x2C; AddConstEx(ID, "TTimer_2C", Address, -1);	}
	return sHandle;
}

static Struct_TMenu()
{
auto	sHandle;
auto	error;
auto	STRUCTURE_NAME;
auto ID, Address;
auto EnumName;
	//     TPersestent,    
auto	sHandle_Obj;
	sHandle_Obj = GetStrucIdByName("TComponent_obj");
	if(sHandle_Obj == -1)
	{	sHandle_Obj = Struct_TComponent();	}


	STRUCTURE_NAME = "TMenu_obj";
	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{	sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{	WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
			return -1;	}
		return sHandle;		}
	AddStrucMember(sHandle, "TComponent",		0x00, FF_STRU, sHandle_Obj, 0x30);
	AddStrucMember(sHandle, "TMenu30",		0x30, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TMenu34",		0x34, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TMenu38",		0x38, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TMenu3C",		0x3C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TMenu40",		0x40, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TMenu44",		0x44, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TMenu48",		0x48, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TMenu4C",		0x4C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TMenu50",		0x50, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TMenu54",		0x54, FF_DWRD, -1, 4); 
	
	EnumName="EnumTMenu";
	ID=AddEnum(0, EnumName, 0);
	if (ID != -1)
	{	Address=0x00; AddConstEx(ID, "TMenu_0", Address, -1);
		Address=0x04; AddConstEx(ID, "TMenu_4", Address, -1);
		Address=0x08; AddConstEx(ID, "TMenu_8", Address, -1);
		Address=0x0C; AddConstEx(ID, "TMenu_C", Address, -1);
		Address=0x10; AddConstEx(ID, "TMenu_10", Address, -1);
		Address=0x14; AddConstEx(ID, "TMenu_14", Address, -1);
		Address=0x18; AddConstEx(ID, "TMenu_18", Address, -1);
		Address=0x1C; AddConstEx(ID, "TMenu_1C", Address, -1);
		Address=0x20; AddConstEx(ID, "TMenu_20", Address, -1);
		Address=0x24; AddConstEx(ID, "TMenu_24", Address, -1);
		Address=0x28; AddConstEx(ID, "TMenu_28", Address, -1);
		Address=0x2C; AddConstEx(ID, "TMenu_2C", Address, -1);
		Address=0x30; AddConstEx(ID, "TMenu_30", Address, -1);
		Address=0x34; AddConstEx(ID, "TMenu_34", Address, -1);		}
	return sHandle;
}

static Struct_TControl()
{
auto	sHandle;
auto	error;
auto	STRUCTURE_NAME;
auto ID, Address;
auto EnumName;
	//     TPersestent,    
auto	sHandle_Obj;
	sHandle_Obj = GetStrucIdByName("TComponent_obj");
	if(sHandle_Obj == -1)
	{	sHandle_Obj = Struct_TComponent();	}


	STRUCTURE_NAME = "TControl_obj";
	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{	sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{	WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
			return -1;	}
		return sHandle;		}
	AddStrucMember(sHandle, "TComponent",		0x00, FF_STRU, sHandle_Obj, 0x30);
	AddStrucMember(sHandle, "TControl30",		0x30, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl34",		0x34, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "WndProc",		0x38, FF_DWRD + FF_0OFF, -1, 4);
	AddStrucMember(sHandle, "TControl3C",		0x3C, FF_DWRD + FF_0OFF, -1, 4);
	AddStrucMember(sHandle, "TControl40",		0x40, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl44",		0x44, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl48",		0x48, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl4C",		0x4C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl50",		0x50, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl54",		0x54, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl58",		0x58, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl5C",		0x5C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl60",		0x60, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl64",		0x64, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TFont",		0x68, FF_DWRD + FF_0OFF, -1, 4); 
	AddStrucMember(sHandle, "TControl6C",		0x6C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl70",		0x70, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl74",		0x74, FF_DWRD + FF_0OFF, -1, 4); 
	AddStrucMember(sHandle, "TControl78",		0x78, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl7C",		0x7C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl80",		0x80, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl84",		0x84, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl88",		0x88, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl8C",		0x8C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl90",		0x90, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl94",		0x94, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "FScalingFlags",	0x98, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl9C",		0x9C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlA0",		0xA0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControlA4",		0xA4, FF_BYTE, -1, 1); 
	AddStrucMember(sHandle, "FRunning",		0xA5, FF_BYTE, -1, 1); 
	AddStrucMember(sHandle, "TControlA8",		0xA8, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlAC",		0xAC, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlB0",		0xB0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControlB4",		0xB4, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "FloatingDockSiteClass",0xB8, FF_DWRD + FF_0OFF, -1, 4); 
	AddStrucMember(sHandle, "TControlBC",		0xBC, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlC0",		0xC0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControlC4",		0xC4, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlC8",		0xC8, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlCC",		0xCC, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlD0",		0xD0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControlD4",		0xD4, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlD8",		0xD8, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlDC",		0xDC, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlE0",		0xE0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControlE4",		0xE4, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlE8",		0xE8, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlEC",		0xEC, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlF0",		0xF0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControlF4",		0xF4, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlF8",		0xF8, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControlFC",		0xFC, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl100",		0x100, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl104",		0x104, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl108",		0x108, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl10C",		0x10C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl110",		0x110, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl114",		0x114, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl118",		0x118, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl11C",		0x11C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl120",		0x120, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl124",		0x124, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl128",		0x128, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl12C",		0x12C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl130",		0x130, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl134",		0x134, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl138",		0x138, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl13C",		0x13C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl140",		0x140, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl144",		0x144, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl148",		0x148, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl14C",		0x14C, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl150",		0x150, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "TControl154",		0x154, FF_DWRD, -1, 4); 
	AddStrucMember(sHandle, "TControl158",		0x158, FF_DWRD, -1, 4); 
	
	EnumName="EnumTControl";
	ID=AddEnum(0, EnumName, 0);
	if (ID != -1)
	{	Address=0x00; AddConstEx(ID, "TControl_0", Address, -1);
		Address=0x04; AddConstEx(ID, "TControl_4", Address, -1);
		Address=0x08; AddConstEx(ID, "TControl_8", Address, -1);
		Address=0x0C; AddConstEx(ID, "TControl_C", Address, -1);
		Address=0x10; AddConstEx(ID, "TControl_10", Address, -1);
		Address=0x14; AddConstEx(ID, "TControl_14", Address, -1);
		Address=0x18; AddConstEx(ID, "TControl_18", Address, -1);
		Address=0x1C; AddConstEx(ID, "TControl_1C", Address, -1);
		Address=0x20; AddConstEx(ID, "TControl_20", Address, -1);
		Address=0x24; AddConstEx(ID, "TControl_24", Address, -1);
		Address=0x28; AddConstEx(ID, "TControl_28", Address, -1);
		Address=0x2C; AddConstEx(ID, "TControl_2C", Address, -1);
		Address=0x30; AddConstEx(ID, "TControl_30", Address, -1);
		Address=0x34; AddConstEx(ID, "TControl_34", Address, -1);
		Address=0x38; AddConstEx(ID, "TControl_38", Address, -1);
		Address=0x3C; AddConstEx(ID, "TControl_3C", Address, -1);
		Address=0x40; AddConstEx(ID, "TControl_40", Address, -1);
		Address=0x44; AddConstEx(ID, "TControl_44", Address, -1);
		Address=0x48; AddConstEx(ID, "TControl_48", Address, -1);
		Address=0x4C; AddConstEx(ID, "TControl_4C", Address, -1);
		Address=0x50; AddConstEx(ID, "TControl_50", Address, -1);
		Address=0x54; AddConstEx(ID, "TControl_54", Address, -1);
		Address=0x58; AddConstEx(ID, "TControl_58", Address, -1);
		Address=0x5C; AddConstEx(ID, "TControl_5C", Address, -1);
		Address=0x60; AddConstEx(ID, "TControl_60", Address, -1);
		Address=0x64; AddConstEx(ID, "TControl_64", Address, -1);
		Address=0x68; AddConstEx(ID, "TControl_68", Address, -1);
		Address=0x6C; AddConstEx(ID, "TControl_6C", Address, -1);
		Address=0x70; AddConstEx(ID, "TControl_70", Address, -1);
		Address=0x74; AddConstEx(ID, "TControl_74", Address, -1);
		Address=0x78; AddConstEx(ID, "TControl_78", Address, -1);
		Address=0x7C; AddConstEx(ID, "TControl_7C", Address, -1);
		Address=0x80; AddConstEx(ID, "TControl_80", Address, -1);
		Address=0x84; AddConstEx(ID, "TControl_84", Address, -1);
		Address=0x88; AddConstEx(ID, "TControl_88", Address, -1);
	}
	return sHandle;
}

static Struct_TApplication()
{
auto	sHandle, sHandle_TObject;
auto	error;
auto	NAME, STRUCTURE_NAME, STRUCTURE_PARENT;
auto ID, Address;
auto EnumName;

// Init Data
	NAME = "TApplication";
	STRUCTURE_PARENT = "TComponent_obj";
	STRUCTURE_NAME = NAME+"_obj";
	EnumName="Enum"+NAME;

	//     ,    
	sHandle_TObject = GetStrucIdByName(STRUCTURE_PARENT);
	if(sHandle_TObject == -1)
	{	sHandle_TObject = Struct_TComponent();	} // ************** !!!!!!!!!!!!!!!

	sHandle = AddStrucEx(-1, STRUCTURE_NAME, 0);
	if(sHandle == -1)
	{	sHandle = GetStrucIdByName(STRUCTURE_NAME);
		if(sHandle == -1)
		{	WarningMessage("Unable to create the " + STRUCTURE_NAME + " structure!\n");
			return -1;	}
		return sHandle;		}
		
	AddStrucMember(sHandle, "pVTBL",		0x00, FF_STRU, sHandle_TObject, 0x30);
	AddStrucMember(sHandle, "FHandle",		0x30, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"34",		0x34, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"38",		0x38, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"3C",		0x3C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"40",		0x40, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, "FMainForm",		0x44, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"48",		0x48, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"4C",		0x4C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"50",		0x50, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"54",		0x54, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"58",		0x58, FF_BYTE, -1, 4);
	AddStrucMember(sHandle, "FShowMainForm",	0x5B, FF_BYTE, -1, 4);
	AddStrucMember(sHandle, NAME+"5C",		0x5C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"60",		0x60, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"64",		0x64, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"68",		0x68, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"6C",		0x6C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"70",		0x70, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"74",		0x74, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"78",		0x78, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"7C",		0x7C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"80",		0x80, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"84",		0x84, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"88",		0x88, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"8C",		0x8C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"90",		0x90, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"94",		0x94, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"98",		0x98, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"9C",		0x9C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"A0",		0xA0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"A4",		0xA4, FF_BYTE, -1, 1);
	AddStrucMember(sHandle, "FRunning",		0xA5, FF_BYTE, -1, 1);
	AddStrucMember(sHandle, NAME+"A8",		0xA8, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"AC",		0xAC, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"B0",		0xB0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"B4",		0xB4, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"B8",		0xB8, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"BC",		0xBC, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"C0",		0xC0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"C4",		0xC4, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"C8",		0xC8, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"CC",		0xCC, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"D0",		0xD0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"D4",		0xD4, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"D8",		0xD8, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"DC",		0xDC, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"E0",		0xE0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"E4",		0xE4, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"E8",		0xE8, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"EC",		0xEC, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"F0",		0xF0, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"F4",		0xF4, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"F8",		0xF8, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"FC",		0xFC, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"100",		0x100, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"104",		0x104, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"108",		0x108, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"10C",		0x10C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"110",		0x110, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"114",		0x114, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"118",		0x118, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"11C",		0x11C, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"120",		0x120, FF_DWRD, -1, 4);
	AddStrucMember(sHandle, NAME+"124",		0x124, FF_DWRD, -1, 4);

//	AddStrucMember(sHandle, "FHandle",			0x30, FF_DWRD, -1, 4);
//	AddStrucMember(sHandle, "FMainForm",			0x44, FF_DWRD, -1, 4);
//	AddStrucMember(sHandle, "FHintColor",			0x5C, FF_DWRD, -1, 4);
//	AddStrucMember(sHandle, "FHintWindow",			0x84, FF_DWRD, -1, 4);
//	AddStrucMember(sHandle, "FShowHint",			0x88, FF_BYTE, -1, 1);
//	AddStrucMember(sHandle, "FTitle",			0x8C, FF_DWRD, -1, 4);
//	AddStrucMember(sHandle, "FHandleCreated",		0xA4, FF_BYTE, -1, 1);
//	AddStrucMember(sHandle, "FRunning",			0xA5, FF_BYTE, -1, 1);
}

//****************************************************************************************
//* TList
//****************************************************************************************

static Parse_TList(addr)
{
auto NameS, StructId, MembOff, Value, pItem, Count, I;
	NameS="TList_obj";
	MakeStruct(addr,NameS);
	StructId = GetStrucIdByName(NameS);
	MembOff = GetMemberOffset(StructId,"Count");
	Count=Dword(addr+MembOff);
	MembOff = GetMemberOffset(StructId,"Items");
	pItem=Dword(addr+MembOff);
	for (I=0;I<Count;I++)
	{
		MakeOffset(pItem);
		pItem=pItem+4;
	}
//	MakeStr(Dword(addr+MembOff),BADADDR);
//	Message(ltoa(Value,16));
}

//****************************************************************************************
//* TComponent
//****************************************************************************************

static Parse_TComponent(addr)
{
auto NameS, NameC, StructId, MembOff, Value;
	NameS="TComponent_obj";
//	addr= ScreenEA();
	MakeStruct(addr,NameS);
	StructId = GetStrucIdByName(NameS);
	
	MembOff = GetMemberOffset(StructId,"FName");
//	MakeStr(Dword(addr+MembOff),BADADDR);
	NameC=GetZStr(Dword(addr+MembOff));
	MakeName(addr,NameC); //    Application	
//	Message(ltoa(Value,16));
	MembOff = GetMemberOffset(StructId,"FComponents");
	Value=Dword(addr+MembOff);
	Parse_TList(Value);
	MakeName(Value,NameC+"_FComponents");
	
	MembOff = GetMemberOffset(StructId,"TComponent14");
	Value=Dword(addr+MembOff);
	Parse_TList(Value);
	MakeName(Value,NameC+"_TComponent14");
}



//****************************************************************************************
//*  
//****************************************************************************************

static IunitEntries()
{
	UnitEntries(ScreenEA());
}

static IInitTable()
{
	InitTable(ScreenEA());
}
static CreateObj() //   
{
// 
	Struct_System();
	
// 
	Struct_TObject();
	Struct_TClassFinder();
	Struct_TList();
	Struct_TPersistent();
	Struct_TComponent();
	Struct_TTimer();
	Struct_TMenu();
	Struct_TControl();
	Struct_TApplication();
	
//	Struct_Stream();
}

//    ---------------------------------------------

//  
static main()
{
//	Struct_TList();
	CreateObj();
}
